
/* unit unformus */

#include <math.h>

#include "unformus.h"
#include "unforfun.h"
#include "unfortim.h"
#include "unfornut.h"
#include "unforpnm.h"

/*
 
 purpose:

    computation of the sun's position

 notes:

   this software is protected by national and international copyright.
   any unauthorized use, reproduction or modificaton is unlawful and
   will be prosecuted. commercial and non-private application of the
   software in any form is strictly prohibited unless otherwise granted
   by the authors.

 (c) 1999 oliver montenbruck, thomas pfleger
*/


/*
 sunpos: computes the sun's ecliptical position using analytical series

 input:

   tmjd      time TDB moment in modified julian day

 <return>:   ecliptic spheric position of the sun
             (in radian radian [au]),
             referred to the ecliptic and equinox of date
*/

void sunnewcomb ( double tmjd , /* ecliptic longitude latitude */
                  double *v , double *b , double *r ) /* range */
{
  double um2,um3,um4,um5,um6 ; /* mean anomalies of the planets */
  double dal,daf,dad ; /* mean arguments of lunar orbit */
  double dt,dl,db,dr ; /* corr. longitude["], latitude["], radius[au] */
  double u,s,c,pi22 ;

  pi22 = 2*M_PI ;
  dt = (tmjd-51544.5)/36525;
  /* mean anomalies of planets in radian */
  um2 = pi22*frac(0.1387306+162.5485917*dt); /* the venus   m2 */
  um3 = pi22*frac(0.9931266+99.9973604*dt);  /* the earth   m3 */
  um4 = pi22*frac(0.0543250+53.1666028*dt);  /* the mars    m4 */
  um5 = pi22*frac(0.0551750+8.4293972*dt);   /* the jupiter m5 */
  um6 = pi22*frac(0.8816500+3.3938722*dt);   /* the saturn  m6 */
  /* mean arguments of lunar orbit in radian */
  dad = pi22*frac(0.8274+1236.8531*dt); /* elongation of the moon    d  d */
  dal = pi22*frac(0.3749+1325.5524*dt); /* mean anomaly of the moon  l  a */
  daf = pi22*frac(0.2591+1342.2278*dt); /* mean argument of latitude f  u */
  /* keplerian terms and perturbations by venus */
  dl = 0.0 ; dr = 0.0; db = 0.0; /* initial nullo corrections */
  u = um3;   /* 1 0 0 */
  c = cos(u); s = sin(u);
  dl = dl-0.22*c+6892.76*s;
  dr = dr-16707.37*c-0.54*s;
  dl = dl+(-0.06*c-17.35*s)*dt;      /* 1 0 1 */
  dr = dr+(+42.04*c-0.15*s)*dt;
  dl = dl+(-0.01*c-0.05*s)*dt*dt; /* 1 0 2 */
  dr = dr+(+0.13*c-0.02*s)*dt*dt;
  u = 2*um3;  /* 2 0 1 */
  c = cos(u); s = sin(u);
  dl = dl+71.98*s;
  dr = dr-139.57*c;
  dl = dl-0.36*s*dt;
  dr = dr+0.70*c*dt;
  u = 3*um3; /* 3 0 0 */
  c = cos(u); s = sin(u);
  dl = dl+1.04*s;
  dr = dr-1.75*c;
  u = -um2;  /* 0-1 0 */
  c = cos(u); s = sin(u);
  dl = dl+0.03*c-0.07*s;
  dr = dr-0.16*c-0.07*s;
  db = db+0.02*c-0.02*s;
  u = um3-um2; /* 1,-1,0 */
  c = cos(u); s = sin(u);
  dl = dl+2.35*c-4.23*s;
  dr = dr-4.75*c-2.64*s;
  u = um3-2*um2; /* 1,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.10*c+0.06*s;
  dr = dr+0.12*c+0.20*s;
  db = db+0.02*c;
  u = 2*um3-um2; /* 2,-1,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.06*c-0.03*s;
  dr = dr+0.20*c-0.01*s;
  db = db+0.01*c-0.09*s;
  u = 2*um3-2*um2;/* 2,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl-4.70*c+2.90*s;
  dr = dr+8.28*c+13.42*s;
  db = db+0.01*c-0.01*s;
  u = 3*um3-2*um2; /* 3,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl+1.80*c-1.74*s;
  dr = dr-1.44*c-1.57*s;
  db = db+0.04*c-0.06*s;
  u = 3*um3-3*um2; /* 3,-3,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.67*c+0.03*s;
  dr = dr+0.11*c+2.43*s;
  db = db+0.01*c;
  u = 4*um3-2*um2; /* 4,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.03*c-0.03*s;
  dr = dr+0.10*c+0.09*s;
  db = db+0.01*c-0.01*s;
  u = 4*um3-3*um2; /* 4,-3,0 */
  c = cos(u); s = sin(u);
  dl = dl+1.51*c-0.40*s;
  dr = dr-0.88*c-3.36*s;
  db = db+0.18*c-0.10*s;
  u = 4*um3-4*um2; /* 4,-4,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.19*c-0.09*s;
  dr = dr-0.38*c+0.77*s;
  u = 5*um3-3*um2; /* 5,-3,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.76*c-0.68*s;
  dr = dr+0.30*c+0.37*s;
  db = db+0.01*c;
  u = 5*um3-4*um2; /* 5,-4,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.14*c-0.04*s;
  dr = dr-0.11*c+0.43*s;
  db = db-0.03*c;
  u = 5*um3-5*um2; /* 5,-5,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.05*c-0.07*s;
  dr = dr-0.31*c+0.21*s;
  u = 6*um3-4*um2; /* 6,-4,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.15*c-0.04*s;
  dr = dr-0.06*c-0.21*s;
  db = db+0.01*c;
  u = 6*um3-5*um2; /* 6,-5,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.03*c-0.03*s;
  dr = dr-0.09*c+0.09*s;
  db = db-0.01*c;
  u = 6*um3-6*um2; /* 6,-6,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.04*s;
  dr = dr-0.18*c+0.02*s;
  u = 7*um3-5*um2; /* 7,-5,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.12*c-0.03*s;
  dr = dr-0.08*c+0.31*s;
  db = db-0.02*c-0.01*s;
  /* perturbations by mars */
  u = um3-um4; /* 1,-1,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.22*c+0.17*s;
  dr = dr-0.21*c-0.27*s;
  u = um3-2*um4; /* 1,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl-1.66*c+0.62*s;
  dr = dr+0.16*c+0.28*s;
  u = 2*um3-2*um4; /* 2,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl+1.96*c+0.57*s;
  dr = dr-1.32*c+4.55*s;
  db = db+0.01*s;
  u = 2*um3-3*um4; /* 2,-3,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.40*c+0.15*s;
  dr = dr-0.17*c+0.46*s;
  u = 2*um3-4*um4; /* 2,-4,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.53*c+0.26*s;
  dr = dr+0.09*c-0.22*s;
  u = 3*um3-3*um4; /* 3,-3,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.05*c+0.12*s;
  dr = dr-0.35*c+0.15*s;
  u = 3*um3-4*um4; /* 3,-4,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.13*c-0.48*s;
  dr = dr+1.06*c-0.29*s;
  db = db+0.01*c;
  u = 3*um3-5*um4; /* 3,-5,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.04*c-0.20*s;
  dr = dr+0.20*c-0.04*s;
  u = 4*um3-4*um4; /* 4,-4,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.03*s;
  dr = dr+0.10*c+0.04*s;
  u = 4*um3-5*um4; /* 4,-5,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.05*c-0.07*s;
  dr = dr+0.20*c+0.14*s;
  u = 4*um3-6*um4; /* 4,-6,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.10*c+0.11*s;
  dr = dr-0.23*c-0.22*s;
  u = 5*um3-7*um4; /* 5,-7,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.05*c;
  dr = dr+0.01*c-0.14*s;
  u = 5*um3-8*um4; /* 5,-8,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.05*c+0.01*s;
  dr = dr-0.02*c+0.10*s;
  /* perturbations by jupiter */
  u = -um3-um5; /*-1,-1,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.01*c+0.07*s;
  dr = dr+0.18*c-0.02*s;
  db = db-0.02*s;
  u = -um5; /* 0,-1,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.31*c+2.58*s;
  dr = dr+0.52*c+0.34*s;
  db = db+0.02*c;
  u = um3-um5; /* 1,-1,0 */
  c = cos(u); s = sin(u);
  dl = dl-7.21*c-0.06*s;
  dr = dr+0.13*c-16.27*s;
  db = db-0.02*s;
  u = um3-2*um5; /* 1,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.54*c-1.52*s;
  dr = dr+3.09*c-1.12*s;
  db = db+0.01*c-0.17*s;
  u = um3-3*um5; /* 1,-3,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.03*c-0.21*s;
  dr = dr+0.38*c-0.06*s;
  db = db-0.02*s;
  u = 2*um3-um5; /* 2,-1,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.16*c+0.05*s;
  dr = dr-0.18*c-0.31*s;
  db = db+0.01*c;
  u = 2*um3-2*um5; /* 2,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.14*c-2.73*s;
  dr = dr+9.23*c+0.48*s;
  u = 2*um3-3*um5; /* 2,-3,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.07*c-0.55*s;
  dr = dr+1.83*c+0.25*s;
  db = db+0.01*c;
  u = 2*um3-4*um5; /* 2,-4,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.02*c-0.08*s;
  dr = dr+0.25*c+0.06*s;
  u = 3*um3-2*um5; /* 3,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.01*c-0.07*s;
  dr = dr+0.16*c+0.04*s;
  u = 3*um3-3*um5; /* 3,-3,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.16*c-0.03*s;
  dr = dr+0.08*c-0.64*s;
  u = 3*um3-4*um5; /* 3,-4,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.04*c-0.01*s;
  dr = dr+0.03*c-0.17*s;
  /* perturbations by saturn */
  u = -um6; /* 0,-1,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.32*s;
  dr = dr+0.01*c;
  u = um3-um6; /* 1,-1,0 */
  c = cos(u); s = sin(u);
  dl = dl-0.08*c-0.41*s;
  dr = dr+0.97*c-0.18*s;
  db = db-0.01*s;
  u = um3-2*um6; /* 1,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.04*c+0.10*s;
  dr = dr-0.23*c+0.10*s;
  u = 2*um3-2*um6; /* 2,-2,0 */
  c = cos(u); s = sin(u);
  dl = dl+0.04*c+0.10*s;
  dr = dr-0.35*c+0.13*s;
  /* difference of earth-moon-barycentre and centre of the earth */
  dl = dl+6.45*sin(dad)-0.42*sin(dad-dal)+0.18*sin(dad+dal)
        +0.17*sin(dad-um3)-0.06*sin(dad+um3);
  dr = dr+30.76*cos(dad)-3.06*cos(dad-dal)+0.85*cos(dad+dal)
        -0.58*cos(dad+um3)+0.57*cos(dad-um3);
  db = db+0.576*sin(daf);
  /* long-periodic perturbations */
  dl = dl+6.40*sin(pi22*(0.6983+0.0561*dt))
        +1.87*sin(pi22*(0.5764+0.4174*dt))
        +0.27*sin(pi22*(0.4189+0.3306*dt))
        +0.20*sin(pi22*(0.3581+2.4814*dt));
  /* ecliptic coordinates ([rad],[au]) */
  *v = pi22*frac(0.7859453+um3/pi22
              +((6191.2+1.1*dt)*dt+dl)/1296.0e3);
  *r = 1.0001398-0.0000007*dt+dr*1.0e-6;
  *b = M_PI/(180.0*3600.0)*db;
}

/*
 minisun: computes the sun's ra and declination using a low precision 
          analytical series

 input:

   tmjd      time in modified julian day

 <return>:   ecliptic spheric position of the sun
             (in radian radian [au]),
             referred to the ecliptic and equinox of date
             but very small accuracy
*/

void simplesun ( double tmjd ,
                 double *v , double *b , double *r )
{
  double dt,u3,pi22 ;

  pi22 = 2*M_PI ;
  dt = (tmjd-51544.5)/36525;
  u3 = pi22*frac(0.993133+99.997361*dt); /* mean anomaly in radian */
  *v = pi22*frac(0.7859453+u3/pi22+(6893.0*sin(u3)  /* radian */
              +72.0*sin(2*u3)+6191.2*dt)/1296.0e3); /* longitude */
  *b = 0.0;      /* latitude nullo */
  *r = 1.0001398 /* range in au */
    +1.0e-6*((-16707.37+42.04*dt)*cos(u3)
              -139.57*cos(2*u3)
              +30.76*cos(pi22*frac(0.8274+1236.8531*dt)));
}


/* 
  position refer to mean equator of date t
  t  modified julian day
  v  longitude radian (lambda)  ecliptic of date
  b  latitude  radian (beta)
  r  distance  astronomical unit
  t  moment in modified julian day
  r  vector of position in refer to mean equator
*/

void brmequpos ( double t , double v , double b , double d , double r[4] )
{
  double ep,se,ce  ;    /* for obliquity */
  double cl,sl,cb,sb ;

  ep = togetepsmean(t); /* obliquity unit unfornut */
  se = sin(ep);
  ce = cos(ep);
  sl = sin(v);
  cl = cos(v);
  sb = sin(b);
  cb = cos(b);
  r[1] = d*cb*cl;            /* from ecliptic of date */
  r[2] = d*(cb*sl*ce-sb*se); /* to   equator  of date */
  r[3] = d*(cb*sl*se+sb*ce);
}

/* position from ecliptic spheric
   to descart refer to standard equator j2000.0
   input
     t  TDB moment in modified julian day
     v  longitude radian (lambda)  ecliptic of date
     b  latitude  radian (beta)
     d  distance  astronomical unit 
   output
     r  vector of position refer to standart equator
*/

void brmfixpos ( double t , double v , double b , double d , double r[4] )
{
  int    i,j ;
  double c[4] ;
  double p[4][4] ;

  brmequpos(t,v,b,d,c);     /* from the ecliptic spheric position */
  clcprecmatr(51544.5,t,p); /* void from unforpnm precession matrix */
  for (  i = 1 ;  i <= 3 ; i++ )
    {
      r[i] = 0.0;
      for (  j = 1 ;  j <= 3 ; j++ )
        r[i] = r[i]+p[j][i]*c[j]; /* refer to fixed equator */
    }
}

/* to obtain heliocentric descart position of the Earth
      in the fixed equator J2000.0 (Celestial Reference System)
   input
     t  TDB moment in modified julian day
   output  in kilometer
     r  vector of position of the Earth refer to standart equator
*/

void earthfixpos ( double t , double r[4] )
{
  double v,b,d ; /* the Sun ecliptical spheric radian AU */

  sunnewcomb(t,&v,&b,&d); /* result longitude latitude range mean ecliptic */

  brmfixpos(t,v,b,d,r); /* result vector r the Sun refer to the Earth AU */
                         // the Earth refer to the Sun 
  r[1] = -astrunit*r[1]; /* x coordinate in km by AU */
  r[2] = -astrunit*r[2]; /* y coordinate in km by AU */
  r[3] = -astrunit*r[3]; /* z coordinate in km by AU */
                         // astrunit value in km from unformus.h
}
