/*
  one string in astorb.dat file
  information for minor planet
  i  integer format
  f  float format
  a  character format
                  columns   example         var  comment
  number  a6          1-6       1           snm  may be or may be not
  name    a11        7-17   Ceres           stn
  temp    a9        18-26
  temp    a15       27-41   E. Bowell
  parm.H  f6.2      42-47     3.34          prh
  parm.G  f6.2      48-53     0.12          prg
  temp    a52      54-105    0.72 848.4 G?      0   0   0   0   0   0 63339 5824
  year    i5      106-110    2003           yea  year  for elements epoch
  month   i2      111-112   12              mon  month for elements epoch
  day     i2      113-114   27              day  day   for elements epoch
  el.M    f11.6   115-125    317.773836     elm  mean anomaly in degree
  el.o    f11.6   126-136     74.068475     elp  agr. of perihelion in degree
  el.O    f11.6   137-147     80.479399     eln  ascending node in degree
  el.i    f10.6   148-157    10.583432      eli  inclination in degree
  el.e    f11.8   158-168    0.07951843     ele  eccentricity
  el.a    f13.8   169-181      2.76595745   ela  semimajor axis in AU
  temp    a86     182-267    20031019 2.2E-02 -1.1E-04 20040212 2.3E-02 ...
*/

/* function to find objects in our square from unitasto.c */

#include <math.h>
#include <conio.h>
#include <stdio.h>
#include <io.h>
#include <fcntl.h>     /* for open text file */
#include <alloc.h>
#include <process.h>
#include <sys\stat.h>
#include <stdlib.h>
#include <string.h>

#include "unitasto.h"
#include "unforrom.h"  /* void fromfixtoterram for matrix */
#include "unforfun.h"  /* simple functions */
#include "unforkep.h"  /* descart position kepler elements */
#include "unfortim.h"  /* function transutctotdt */
#include "unformus.h"  /* function earthfixpos */
#include "unastvam.h"  /* function togetmau */
#include "unforcoo.h"  /* void sphericoorindegree */
#include "unfordat.h"  /* function fromdatetomjd */
#include "unfortoh.h"  /* degree to civil form HH MM SS or +-DD ' " */
#include "unsquare.h"  /* function int booyes in square */

/*
   to obtain
      heliocentric J2000 position
      of the observatory in kilometer
   input
      tc     current moment in modified julian day
      xt[4]  geocentric terrestrial observatory position in km
   output
      xh[4]  heliocentric J2000 station position kilometer
*/

void  obtosun ( double  tc ,      // current moment mjd
                double  xt[4] ,   // geoterra position station km
                double  xh[4] )   // output : observatory to the Sun km
{
  double  xg[4] ;  /* geocentric position of the observatory  J2000  km */
  double  xe[4] ;  /* heliocentric position of the Earth in J2000.0  km */

  double  ft[4][4] ;  /* matrix to change from fixed to terrestrial system */
  double  tf[4][4] ;  /* matrix to change from terrestrial to fixed system */

  double  tdb ; /* time dynamic barycentric in mjd */

  /* matrix to change from terrestrial to celestial system */

  fromfixtoterram (tc,0.0e0,&tdb,ft) ; /* unforrom.c result matrix ft */
  transpmatr (ft,tf) ; /* unforfun.c matrix from terra to fixed equator */

  /* to change observatory geocentric position to celestial system J2000 */

  multmatrvec (tf,xt,xg) ; /* void unforfun position in km geocentric */

                         /* heliocentric position in km tfind(utc)->tdb */
  earthfixpos (tdb,xe) ; /* unformus the Earth refer to the Sun J2000.0 */

  xh[1] = xe[1]+xg[1] ;  /* heliocentric J2000 x position observatory km */
  xh[2] = xe[2]+xg[2] ;  /* heliocentric J2000 y position observatory km */
  xh[3] = xe[3]+xg[3] ;  /* heliocentric J2000 z position observatory km */
}


/*
 procedure to get from string
 osculating Kepler elements:
 M mean anomaly in degree
 o argument of perihelion in degree
 O ascending node in degree
 i inclination in degree
 e eccentricity
 a semi-major axis in astronomical unit
*/

void elemfromstr ( char   stelm[1] ,
                   char    snm[1] ,    // number         a6      1 -   6
                   char    stn[1] ,    // name           a11     7 -  17
                   double  *prh ,      // parm.H         f6.2   42 -  47
                   double  *prg ,      // parm.G         f6.2   48 -  53
                   int     *year ,     // year           i4    107 - 110
                   int     *month ,    // month          i2    111 - 112
                   int     *day ,      // day            i2    113 - 114
                   double  *elm ,      // element M      f11.6 115 - 125
                   double  *elp ,      // element o      f11.6 126 - 136
                   double  *eln ,      // element O      f11.6 137 - 147
                   double  *eli ,      // element i      f10.6 148 - 157
                   double  *ele ,      // element e      f11.8 158 - 168
                   double  *ela )      // element a      f13.8 169 - 181
{
  int   i ;        // counts
  char  c[200] ;   // current string buffer

  /* number snm 6 bytes 1 - 6 */
  for ( i = 1 ; i <=6 ; i++ )  // 6 bytes
    snm[i-1] = stelm[i-1] ;    // current byte to our string
  snm[6] = '\0' ; // to close string by nullo

  /* name stn 11 bytes 7 - 17 */
  for ( i = 7 ; i <=17 ; i++ )  // 11 bytes
    stn[i-7] = stelm[i-1] ;    // current byte to our string
  stn[11] = '\0' ; // to close string by nullo

  // *prh   parm.H       f6.2   42 -  47
  for ( i = 42 ; i <=47 ; i++ )  // 6 bytes
    c[i-42] = stelm[i-1] ;
  c[6] = '\0' ; // to close string by nullo
  *prh = atof(c) ;

  // *prg   parm.G       f6.2   42 -  47
  for ( i = 48 ; i <=53 ; i++ )  // 6 bytes
    c[i-48] = stelm[i-1] ;
  c[6] = '\0' ; // to close string by nullo
  *prg = atof(c) ;

  // year           i4    107 - 110
  for ( i = 107 ; i <=110 ; i++ )  // 4 bytes
    c[i-107] = stelm[i-1] ;
  c[4] = '\0' ; // to close string by nullo
  *year = atoi(c) ;

  // month           i2    111 - 112
  for ( i = 111 ; i <=112 ; i++ )  // 2 bytes
    c[i-111] = stelm[i-1] ;
  c[2] = '\0' ; // to close string by nullo
  *month = atoi(c) ;

  // day           i2    113 - 114
  for ( i = 113 ; i <=114 ; i++ )  // 2 bytes
    c[i-113] = stelm[i-1] ;
  c[2] = '\0' ; // to close string by nullo
  *day = atoi(c) ;

  // element M      f11.6 115 - 125
  for ( i = 115 ; i <=125 ; i++ )  // 11 bytes
    c[i-115] = stelm[i-1] ;
  c[11] = '\0' ; // to close string by nullo
  *elm = atof(c) ;  // mean anomaly in degree

  // element o      f11.6 126 - 136
  for ( i = 126 ; i <=136 ; i++ )  // 11 bytes
    c[i-126] = stelm[i-1] ;
  c[11] = '\0' ; // to close string by nullo
  *elp = atof(c) ;  // argument of perihalion in degree

  // element O      f11.6 137 - 147
  for ( i = 137 ; i <=147 ; i++ )  // 11 bytes
    c[i-137] = stelm[i-1] ;
  c[11] = '\0' ; // to close string by nullo
  *eln = atof(c) ;  // the ascending node in degree

  // element i      f10.6 148 - 157
  for ( i = 148 ; i <=157 ; i++ )  // 10 bytes
    c[i-148] = stelm[i-1] ;
  c[10] = '\0' ; // to close string by nullo
  *eli = atof(c) ;  // inclination in degree

  // element e      f11.8 158 - 168
  for ( i = 158 ; i <=168 ; i++ )  // 11 bytes
    c[i-158] = stelm[i-1] ;
  c[11] = '\0' ; // to close string by nullo
  *ele = atof(c) ;  // eccentricity

  // element a      f13.8 169 - 181
  for ( i = 169 ; i <=181 ; i++ )  // 13 bytes
    c[i-169] = stelm[i-1] ;
  c[13] = '\0' ; // to close string by nullo
  *ela = atof(c) ;  // semi-major axis in AU

}

/* minor planet J2000 position in kilometer
   refer to the observatory on the surface of the Earth with delay */
                                    // input :
void  astosta ( double  elepo ,     // kepler elements epoch
                double  pose[4] ,   // kepler positional elements
                double  ange[4] ,   // kepler angle elements
                double  tfind ,     // moment in modified julian day
                double  xh[4] ,     // observatory refer to the Sun km
                double  sunp[4] ,   // output refer to the Sun
                double  posp[4] )   // output : position minor planet
{
                           // for the minor planet  km km/s
  double vv[4] ;           // heliocentric  velosity J2000.0
  int    i , iter ;        // simple count for iteration
  double dr , dt , tcur ;  // distance time delay time with delay       

  /* asteroid from Kepler ecliptic elements
     with epoch  elepo  to our moment tfind
     to descart equator J2000.0 position velosity in km km/s */

  fromkeplertokarm(elepo,pose,ange,tfind,sunp,vv); /* unforkep */

  for ( iter = 1 ; iter <= 2 ; iter++ )  // it is enough
    {                                    // only two iterations

      for ( i = 1 ; i <= 3 ; i++ )  // for three coordinates in km
        posp[i] = sunp[i]-xh[i] ;   // minor planet to observatory
       
                                    // from asteroid to observatory distance
      dr = sqrt (sqr(posp[1])+sqr(posp[2])+sqr(posp[3])) ; // sqr unforfun
      dt = (dr/299792.458)/86400.0 ; // time delay in day from distance in km

      tcur = tfind-dt ; // moment with delay for minor planet position

      /* asteroid from Kepler ecliptic elements
         with epoch  elepo  to our moment tfind
         to descart equator J2000.0 position velosity in km km/s */

      fromkeplertokarm(elepo,pose,ange,tcur,sunp,vv); /* unforkep */

    }
                                // with delay
  for ( i = 1 ; i <= 3 ; i++ )  // for three coordinates in km
    posp[i] = sunp[i]-xh[i] ;     // minor planet to observatory result

}

/* general procedure to calculate position of the asteroid in square */

void  astinsquare ( double  xpos ,   /* observatory terra x in km */
                    double  ypos ,   /* observatory terra y in km */
                    double  zpos ,   /* observatory terra z in km */
                    double  tfind ,  /* moment in mjd to calculate */
                    double  alphc ,  /* center R.A. in degree */
                    double  deltc ,  /* center declination in degree */
                    double  halfs ,  /* half of square in degree */
                    double  vmago )  /* value to limit magnitude */
{
  double  xt[4] ;  /* geocentric terrestrial observatoty position in km */
  double  xh[4] ;  /* heliocentric position of the station in J2000  km */

  int     txtelm ;     /* text file with elements astorb.dat for example */
  char    stelm[271] ;   /* length of one string in the text file */

  long    ic , nc ;  // count to read file astorb.dat
                     // variables to read file astorb.dat
  char    snm[12] ;  // number         a6      1 -   6
  char    stn[12] ;  // name           a11     7 -  17
  double  prh ;      // parm.H         f6.2   42 -  47
  double  prg ;      // parm.G         f6.2   48 -  53
  int     year ;     // year           i4    107 - 110
  int     month ;    // month          i2    111 - 112
  int     day ;      // day            i2    113 - 114
  double  elm ;      // element M      f11.6 115 - 125
  double  elp ;      // element o      f11.6 126 - 136
  double  eln ;      // element O      f11.6 137 - 147
  double  eli ;      // element i      f10.6 148 - 157
  double  ele ;      // element e      f11.8 158 - 168
  double  ela ;      // element a      f13.8 169 - 181

  double  elepo ;             /* epoch of the elements in mjd */
  double  pose[4],ange[4] ;   /* kepler elements initial */

  double  sunp[4] ; // minor planet refer to the Sun J2000 in km
  double  posp[4] ; // minor planet refer to observatory J2000 in km
                                  // J2000 system  in degree km
  double  alpha , delta , range ; // spheric refer to observatory

  double  ramot , dcmot ; // motion for r.a. decl. in arcsec per hour

  double  vau ; // current value stellar magnitude

  int     khour , kminu ; // civil form for R.A. HH MM of time
  int     ldegr , lminu ; // civil form for Decl DD mm of arc
  double  rasec , desec ; // civil form second of time of arc
  char    chars ;         // signum for declination '+' or '-'

  /* variables to obtain
     motion for r.a. and decl. in arcsec per hour */

  double  dstep = 1.0/1440.0 ;  // one minute in day
  double  tu,ua,ud ;            // moment r.a. decl. one minute before
  double  tz,za,zd ;            // moment r.a. decl. one minute after
  double  xu[4],xz[4] ;         // heliocetric observatory position


  xt[1] = xpos ; /* observatory terrestrial x position to array in km */
  xt[2] = ypos ; /* observatory terrestrial y position to array in km */
  xt[3] = zpos ; /* observatory terrestrial z position to array in km */

                          // observatory refer to the Sun
  obtosun (tfind,xt,xh) ; // here above position in km J2000 system

  /* a try to open text file with elements */

  txtelm = open("d:\\delphip\\hazaster\\hazasdat\\astorb.dat",
                O_RDONLY|O_TEXT , S_IREAD) ; /* fcntl.h io.h */

  /* to read information for all minor planets from thios text file */

  ic = 0 ; /* initial nullo count */
  nc = 0 ; /* initial nullo count */

  /* simple title to print results */

  printf("\nobject designation       r.a.         decl.     v       motion/hr");
  printf("\n                       h  m  s       o  '              ");
  printf("r.a.   decl.\n");
  printf("\n");

  do         /* to read record by record from file to string */
  {
    nc = nc + 1 ; /* the next string */

    /* a try to read one string */

    read (txtelm,stelm,268) ; /* information to string */

    stelm[267] = '\0' ; // the end of string

    /* function above in this unit
       to get value of the elements from string that read */

    elemfromstr (stelm,snm,stn,                          /* number name */
                       &prh,&prg,                        /* param. H G */
                       &year,&month,&day,                /* date */
                       &elm,&elp,&eln,&eli,&ele,&ela) ;  /* elements */

                      /* elements epoch modified julian day */
    elepo = fromdatetomjd ( day,month,year,0,0,0.0 ) ; /* unfordat */
                      /* elements to array for unforkep unit */
    pose[1] =  ela ;  /* semi-major axis AU */
    pose[2] =  ele ;  /* eccentricity */
    pose[3] =  eli ;  /* inclination in degree */

    ange[1] =  eln ;  /* the ascending node in degree */
    ange[2] =  elp ;  /* argument of perigei in degree */
    ange[3] =  elm ;  /* mean anomaly in degree */

    /* minor planet J2000 position in km
       sunp[4] refer to the Sun in J2000 system
       posp[4] refer to the observatory with delay */

    astosta (elepo,pose,ange,tfind,xh,sunp,posp) ; // void simple here above

    vau = togetvau(prh,prg,sunp,posp) ; // unastvam.h stellar magnitude
                       // minor planet position to spheric coordinates
    sphericoorindegree (posp,&alpha,&delta,&range) ; // unforcoo

    /* to check for object to be or not in our square */

    if ( booyes(alpha,delta,vau,alphc,deltc,halfs,vmago) )  // unsquare.h
      {
        ic = ic + 1 ; /* the next string with our object in square */

        /* result object from observatory in J2000.0 system
           right ascension declination angle values to civil form */

        prhourangle (alpha,&khour,&kminu,&rasec) ; // unfortoh
        prdegrangle (delta,&chars,&ldegr,&lminu,&desec) ; // unfortoh

        /* to obtain motion in arcsec per hour
           for right ascension and declination */

        tu = tfind-dstep ;   // moment one minute before
                             // for the observatory
        obtosun (tu,xt,xu) ; // heliocentric position J2000 kilometer
                             // for the object heliocentric in km
        astosta (elepo,pose,ange,tu,xu,sunp,posp) ; // one minute before
        // minor planet refer to observatory to spheric coordinates
        sphericoorindegree (posp,&ua,&ud,&range) ; // unforcoo degree

        tz = tfind+dstep ;   // moment one minute after
                             // for the observatory
        obtosun (tz,xt,xz) ; // heliocentric position J2000 kilometer
                             // for the object heliocentric 
        astosta (elepo,pose,ange,tz,xz,sunp,posp) ; // one minute after
                       // minor planet position to spheric coordinates
        sphericoorindegree (posp,&za,&zd,&range) ; // unforcoo degree
                       // change (motion) for right ascension (r.a.)
        ramot = 3600.0*((za-ua)/(2.0*dstep))/24.0; // arcsec per hour
                       // change (motion) for declination     (decl)
        dcmot = 3600.0*((zd-ud)/(2.0*dstep))/24.0; // arcsec per hour

        printf ("|%s|%s| %2i %2i %5.2f  %1c%2i %2i %4.1f %5.1f %7.1f %7.1f\n",
                 snm,stn,               // number name
                 khour,kminu,rasec,     // R.A. civil form
                 chars,                 // signum for declination
                 ldegr,lminu,desec,     // Decl civil form
                 vau,                   // stellar magnitude
                 ramot,dcmot ) ;        // motion r.a. decl
      }

  }                         // operator do
  while ( !eof (txtelm) ) ; // operator do

  printf ("\n number of objects %8i  from  %8i  in list\n",ic,nc);

  close (txtelm) ;  /* that is all with this text file io.h */

}
